@@ -27,6 +27,7 @@
 #include <config.h>
 #endif
 
+#include <libgnome/gnome-i18n.h>
 #include <string.h>
 #include <gnome.h>
 #include <bonobo.h>
@@ -443,7 +444,10 @@ editor_set_format (GtkHTMLControlData *cd, gboolean format_html)
 
 static enum {
 	PROP_EDIT_HTML,
-	PROP_HTML_TITLE
+	PROP_HTML_TITLE,
+	PROP_INLINE_SPELLING,
+	PROP_MAGIC_LINKS,
+	PROP_MAGIC_SMILEYS
 } EditorControlProps;
 
 static void
@@ -462,6 +466,15 @@ editor_get_prop (BonoboPropertyBag *bag,
 	case PROP_HTML_TITLE:
 		BONOBO_ARG_SET_STRING (arg, gtk_html_get_title (cd->html));
 		break;
+	case PROP_INLINE_SPELLING:
+		BONOBO_ARG_SET_BOOLEAN (arg, gtk_html_get_inline_spelling (cd->html));
+		break;
+	case PROP_MAGIC_LINKS:
+		BONOBO_ARG_SET_BOOLEAN (arg, gtk_html_get_magic_links (cd->html));
+		break;
+	case PROP_MAGIC_SMILEYS:
+		BONOBO_ARG_SET_BOOLEAN (arg, gtk_html_get_magic_smileys (cd->html));
+		break;
        	default:
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
 		break;
@@ -485,6 +498,15 @@ editor_set_prop (BonoboPropertyBag *bag,
 	case PROP_HTML_TITLE:
 		gtk_html_set_title (cd->html, BONOBO_ARG_GET_STRING (arg));
 		break;
+	case PROP_INLINE_SPELLING:
+		gtk_html_set_inline_spelling (cd->html, BONOBO_ARG_GET_BOOLEAN (arg));
+		break;
+	case PROP_MAGIC_LINKS:
+		gtk_html_set_magic_links (cd->html, BONOBO_ARG_GET_BOOLEAN (arg));
+		break;
+	case PROP_MAGIC_SMILEYS:
+		gtk_html_set_magic_smileys (cd->html, BONOBO_ARG_GET_BOOLEAN (arg));
+		break;
 	default:
 		bonobo_exception_set (ev, ex_Bonobo_PropertyBag_NotFound);
 		break;
@@ -509,6 +531,7 @@ editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 	html_widget = gtk_html_new ();
 	gtk_html_load_empty (GTK_HTML (html_widget));
 	gtk_html_set_editable (GTK_HTML (html_widget), TRUE);
+	gtk_html_set_animate (GTK_HTML (html_widget), FALSE);
 
 	cd = gtk_html_control_data_new (GTK_HTML (html_widget), vbox);
 	g_signal_connect (control, "destroy", G_CALLBACK (control_destroy), cd);
@@ -539,6 +562,36 @@ editor_control_construct (BonoboControl *control, GtkWidget *vbox)
 
 	CORBA_free (def);
 
+	def = bonobo_arg_new (BONOBO_ARG_BOOLEAN);
+	BONOBO_ARG_SET_BOOLEAN (def, gtk_html_get_inline_spelling (GTK_HTML (html_widget)));
+
+	bonobo_property_bag_add (pb, "InlineSpelling", PROP_INLINE_SPELLING,
+				 BONOBO_ARG_BOOLEAN, def,
+				 "Include spelling errors inline", 
+				 0);
+
+	CORBA_free (def);
+
+	def = bonobo_arg_new (BONOBO_ARG_BOOLEAN);
+	BONOBO_ARG_SET_BOOLEAN (def, gtk_html_get_magic_links (GTK_HTML (html_widget)));
+
+	bonobo_property_bag_add (pb, "MagicLinks", PROP_MAGIC_LINKS,
+				 BONOBO_ARG_BOOLEAN, def,
+				 "Recognize links in text and replace them", 
+				 0);
+
+	CORBA_free (def);
+
+	def = bonobo_arg_new (BONOBO_ARG_BOOLEAN);
+	BONOBO_ARG_SET_BOOLEAN (def, gtk_html_get_magic_smileys (GTK_HTML (html_widget)));
+
+	bonobo_property_bag_add (pb, "MagicSmileys", PROP_MAGIC_SMILEYS,
+				 BONOBO_ARG_BOOLEAN, def,
+				 "Recognize smileys in text and replace them", 
+				 0);
+
+	CORBA_free (def);
+
 	def = bonobo_arg_new (BONOBO_ARG_STRING);
 	BONOBO_ARG_SET_STRING (def, "");
 
