@@ -21,7 +21,8 @@
 
 #include <config.h>
 #include <gtk/gtkmain.h>
-#include "gtkhtml-im.h"
+#include "gtkhtml.h"
+#include "gtkhtml-private.h"
 #include "htmlcursor.h"
 #include "htmlengine.h"
 #include "htmlengine-edit-cursor.h"
@@ -31,7 +32,6 @@
 #include "htmlobject.h"
 #include "htmltable.h"
 
-
 #define BLINK_TIMEOUT 500
 
 static GdkColor table_stipple_active_on      = { 0, 0,      0,      0xffff };
@@ -39,15 +39,14 @@ static GdkColor table_stipple_active_off     = { 0, 0xffff, 0xffff, 0xffff };
 static GdkColor table_stipple_non_active_on  = { 0, 0xaaaa, 0xaaaa, 0xaaaa };
 static GdkColor table_stipple_non_active_off = { 0, 0xffff, 0xffff, 0xffff };
 
-static GdkColor cell_stipple_active_on      = { 0, 0xffff, 0xffff, 0 };
-static GdkColor cell_stipple_active_off     = { 0, 0,      0,      0 };
-static GdkColor cell_stipple_non_active_on  = { 0, 0xaaaa, 0xaaaa, 0xaaaa };
+static GdkColor cell_stipple_active_on      = { 0, 0x7fff, 0x7fff, 0 };
+static GdkColor cell_stipple_active_off     = { 0, 0xffff, 0xffff, 0xffff };
+static GdkColor cell_stipple_non_active_on  = { 0, 0x7aaa, 0x7aaa, 0x7aaa };
 static GdkColor cell_stipple_non_active_off = { 0, 0xffff, 0xffff, 0xffff };
 
 static GdkColor image_stipple_active_on      = { 0, 0xffff, 0,      0 };
-static GdkColor image_stipple_active_off     = { 0, 0,      0,      0 };
+static GdkColor image_stipple_active_off     = { 0, 0xffff, 0xffff, 0xffff };
 
-
 void
 html_engine_hide_cursor  (HTMLEngine *engine)
 {
@@ -73,15 +72,6 @@ html_engine_show_cursor  (HTMLEngine *engine)
 	}
 }
 
-static inline void
-move_rect (HTMLEngine *engine, gint *x1, gint *y1, gint *x2, gint *y2)
-{
-	*x1 = *x1 + engine->leftBorder - engine->x_offset;
-	*y1 = *y1 + engine->topBorder - engine->y_offset;
-	*x2 = *x2 + engine->leftBorder - engine->x_offset;
-	*y2 = *y2 + engine->topBorder - engine->y_offset;
-}
-
 static gboolean
 clip_rect (HTMLEngine *engine, gint x, gint y, gint width, gint height, gint *x1, gint *y1, gint *x2, gint *y2)
 {
@@ -108,21 +98,19 @@ draw_cursor_rectangle (HTMLEngine *e, gint x1, gint y1, gint x2, gint y2,
 {
 	GdkGC *gc;
 	GdkColor color;
-	gint8 dashes [2] = { 2, 2 };
-
-	move_rect (e, &x1, &y1, &x2, &y2);
+	gint8 dashes [2] = { 1, 3 };
 
 	gc = gdk_gc_new (e->window);
 	color = *on_color;
-	gdk_color_alloc (gdk_window_get_colormap (e->window), &color);
+	gdk_rgb_find_color (gdk_drawable_get_colormap (e->window), &color);
 	gdk_gc_set_foreground (gc, &color);
 	color = *off_color;
-	gdk_color_alloc (gdk_window_get_colormap (e->window), &color);
+	gdk_rgb_find_color (gdk_drawable_get_colormap (e->window), &color);
 	gdk_gc_set_background (gc, &color);
 	gdk_gc_set_line_attributes (gc, 1, GDK_LINE_DOUBLE_DASH, GDK_CAP_ROUND, GDK_JOIN_ROUND);
 	gdk_gc_set_dashes (gc, offset, dashes, 2);
 	gdk_draw_rectangle (e->window, gc, 0, x1, y1, x2 - x1, y2 - y1);
-	gdk_gc_unref (gc);
+	g_object_unref (gc);
 }
 
 static gint cursor_enabled = TRUE;
@@ -131,7 +119,7 @@ static inline void
 refresh_under_cursor (HTMLEngine *e, HTMLCursorRectangle *cr, gboolean *enabled)
 {
 	*enabled = cursor_enabled = FALSE;
-	html_engine_draw (e, e->leftBorder + cr->x1 - e->x_offset, e->topBorder + cr->y1 - e->y_offset,
+	html_engine_draw (e, cr->x1, cr->y1,
 			  cr->x2 - cr->x1 + 1, cr->y2 - cr->y1 + 1);
 	*enabled = cursor_enabled = TRUE;
 }
@@ -275,6 +263,7 @@ html_engine_draw_cursor_in_area (HTMLEngine *engine,
 	HTMLObject *obj;
 	guint offset;
 	gint x1, y1, x2, y2;
+	GdkRectangle pos;
 
 	g_assert (engine->editable);
 
@@ -294,19 +283,23 @@ html_engine_draw_cursor_in_area (HTMLEngine *engine,
 	offset = engine->cursor->offset;
 
 	if (width < 0 || height < 0) {
-		width = engine->width;
-		height = engine->height;
+		width = html_engine_get_doc_width (engine);
+		height = html_engine_get_doc_height (engine);
 		x = 0;
 		y = 0;
 	}
 
+	
 	html_object_get_cursor (obj, engine->painter, offset, &x1, &y1, &x2, &y2);
-	move_rect (engine, &x1, &y1, &x2, &y2);
+	
+	pos.x = x1; 
+	pos.y = y1;
+	pos.width = x2 - x1;
+	pos.height = x2 - x1;
+	gtk_im_context_set_cursor_location (GTK_HTML (engine->widget)->priv->im_context, &pos);
+
 	if (clip_rect (engine, x, y, width, height, &x1, &y1, &x2, &y2)) {
 		gdk_draw_line (engine->window, engine->invert_gc, x1, y1, x2, y2);
-#ifdef GTK_HTML_USE_XIM
-		gtk_html_im_position_update (engine->widget, x2, y2);
-#endif
 	}
 }
 
