@@ -37,6 +37,7 @@
 #include "htmlcolorset.h"
 #include "htmlembedded.h"
 #include "htmlengine.h"
+#include "htmltextslave.h"
 #include "gtkhtml-embedded.h"
 
 static HTMLPainterClass *parent_class = NULL;
@@ -85,15 +86,15 @@ html_gdk_painter_text_itemize_and_prepare_glyphs (HTMLGdkPainter *painter, Pango
 
 	if (items && items->data) {
 		PangoItem *item;
-		PangoGlyphString *str;
 		GList *il;
+		const gchar *end;
 
 		*glyphs = NULL;
 		for (il = items; il; il = il->next) {
-			str = pango_glyph_string_new ();
 			item = (PangoItem *) il->data;
-			pango_shape (text + item->offset, item->length, &item->analysis, str);
-			*glyphs = g_list_prepend (*glyphs, str);
+			end = g_utf8_offset_to_pointer (text, item->num_chars);
+			*glyphs = html_get_glyphs_non_tab (*glyphs, item, text, end - text, item->num_chars);
+			text = end;
 		}
 		*glyphs = g_list_reverse (*glyphs);
 	} else
@@ -148,8 +149,19 @@ text_width (HTMLGdkPainter *painter, PangoFontDescription *desc, const gchar *te
 	return PANGO_PIXELS (width);
 }
 
+static inline GList *
+shift_items (GList *items, gint byte_offset)
+{
+	PangoItem *item;
+
+	while (items && (item = (PangoItem *)items->data) && item->offset + item->length <= byte_offset)
+		items = items->next;
+
+	return items;
+}
+
 static void
-text_size (HTMLGdkPainter *painter, PangoFontDescription *desc, const gchar *text, gint bytes, GList *items, GList *glyphs, gint *width, gint *asc, gint *dsc)
+text_size (HTMLGdkPainter *painter, PangoFontDescription *desc, const gchar *text, gint bytes, GList *items, GList *glyphs, gint start_byte_offset, gint *width, gint *asc, gint *dsc)
 {
 	gboolean temp_items = FALSE;
 	if (!items) {
@@ -161,12 +173,27 @@ text_size (HTMLGdkPainter *painter, PangoFontDescription *desc, const gchar *tex
 	if (items && items->data && glyphs) {
 		GList *gl, *il;
 		PangoRectangle log_rect;
+		PangoItem *item;
+		PangoGlyphString *str;
+		const gchar *c_text = text;
+		gint c_bytes;
 
-		for (gl = glyphs, il = items; gl && il; gl = gl->next, il = il->next) {
-			pango_glyph_string_extents ((PangoGlyphString *) gl->data, ((PangoItem *) il->data)->analysis.font, NULL, &log_rect);
+		il = shift_items (items, start_byte_offset);
+
+		c_bytes = 0;
+		for (gl = glyphs; gl && c_bytes < bytes; gl = gl->next) {
+			str = (PangoGlyphString *) gl->data;
+			item = (PangoItem *) il->data;
+			pango_glyph_string_extents (str, item->analysis.font, NULL, &log_rect);
 			*width += PANGO_PIXELS (log_rect.width);
 			*asc = MAX (*asc, PANGO_PIXELS (PANGO_ASCENT (log_rect)));
 			*dsc = MAX (*dsc, PANGO_PIXELS (PANGO_DESCENT (log_rect)));
+
+			c_text = g_utf8_offset_to_pointer (c_text, str->num_glyphs);
+			if (*text == '\t')
+				c_text ++;
+			c_bytes = c_text - text;
+			il = shift_items (il, start_byte_offset + c_bytes);
 		}
 	}
 
@@ -181,17 +208,19 @@ text_size (HTMLGdkPainter *painter, PangoFontDescription *desc, const gchar *tex
 static HTMLFont *
 alloc_font (HTMLPainter *painter, gchar *face, gdouble size, gboolean points, GtkHTMLFontStyle style)
 {
-	PangoFontDescription *desc;
-	
+	PangoFontDescription *desc = NULL;
+
 	if (face) {
 		desc = pango_font_description_from_string (face);
-	} else {
+	}
+
+	if (!desc || !pango_font_description_get_family (desc)) {
+		if (desc)
+			pango_font_description_free (desc);
+
 		desc = pango_font_description_copy (((HTMLGdkPainter *)painter)->style->font_desc);
 	}
-	
-	if (style & GTK_HTML_FONT_STYLE_FIXED) {
-		pango_font_description_set_family (desc, "Monospace");
-	}
+
 	pango_font_description_set_size (desc, size * PANGO_SCALE);
 	pango_font_description_set_style (desc, style & GTK_HTML_FONT_STYLE_ITALIC ? PANGO_STYLE_ITALIC : PANGO_STYLE_NORMAL);
 	pango_font_description_set_weight (desc, style & GTK_HTML_FONT_STYLE_BOLD ? PANGO_WEIGHT_BOLD : PANGO_WEIGHT_NORMAL);
@@ -883,26 +912,32 @@ fill_rect (HTMLPainter *painter,
 }
 
 static gint
-draw_spell_error (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, GList *items, GList *glyphs)
+draw_spell_error (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, GList *items, GList *glyphs, gint start_byte_offset)
 {
-	PangoFontDescription *desc;
 	HTMLGdkPainter *gdk_painter;
 	GdkGCValues values;
 	gchar dash [2];
 	GList *gl, *il;
 	PangoRectangle log_rect;
+	PangoGlyphString *str;
 	gint width;
+	const gchar *c_text = text;
 
 	if (!items || !glyphs)
-		return;
+		return 0;
 
 	gdk_painter = HTML_GDK_PAINTER (painter);
 
 	x -= gdk_painter->x1;
 	y -= gdk_painter->y1;
 
-	for (gl = glyphs, il = items; gl && il; gl = gl->next, il = il->next)
-		pango_glyph_string_extents ((PangoGlyphString *) gl->data, ((PangoItem *) il->data)->analysis.font, NULL, &log_rect);
+	il = shift_items (items, start_byte_offset);
+	for (gl = glyphs; gl; gl = gl->next) {
+		str = (PangoGlyphString *) gl->data;
+		pango_glyph_string_extents (str, ((PangoItem *) il->data)->analysis.font, NULL, &log_rect);
+		c_text = g_utf8_offset_to_pointer (c_text, str->num_glyphs);
+		il = shift_items (il, start_byte_offset + (c_text - text));
+	}
 
 	width = PANGO_PIXELS (log_rect.width);
 
@@ -937,13 +972,13 @@ draw_embedded (HTMLPainter * p, HTMLEmbedded *o, gint x, gint y)
 	}
 }
 
-static void
-draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, GList *items, GList *glyphs)
+static gint
+draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, GList *items, GList *glyphs, gint start_byte_offset)
 {
 	HTMLGdkPainter *gdk_painter;
 	PangoFontDescription *desc;
 	gboolean temp_items = FALSE;
-	gint blen;
+	gint blen, width = 0;
 
 	if (len == -1)
 		len = g_utf8_strlen (text, -1);
@@ -957,25 +992,31 @@ draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, GL
 	blen = g_utf8_offset_to_pointer (text, len) - text;
 	if (!items) {
 		items = html_gdk_painter_text_itemize_and_prepare_glyphs (gdk_painter, desc, text, blen, &glyphs);
+		start_byte_offset = 0;
 		temp_items = TRUE;
 	}
 	if (items && items->data) {
 		PangoGlyphString *str;
 		GList *gl, *il;
-		guint i, width = 0;
+		guint i, char_offset = 0;
+		const gchar *c_text = text;
 
-		for (gl = glyphs, il = items; il && gl; gl = gl->next, il = il->next) {
+		il = shift_items (items, start_byte_offset);
+		for (gl = glyphs; gl && char_offset < len; gl = gl->next) {
 			str = (PangoGlyphString *) gl->data;
 			gdk_draw_glyphs (gdk_painter->pixmap, gdk_painter->gc, ((PangoItem *) il->data)->analysis.font, x + width, y, str);
 			for (i=0; i < str->num_glyphs; i ++)
 				width += PANGO_PIXELS (str->glyphs [i].geometry.width);
+			c_text = g_utf8_offset_to_pointer (c_text, str->num_glyphs);
+			il = shift_items (il, start_byte_offset + (c_text - text));
+			char_offset += str->num_glyphs;
 		}
 	}
 
 	if (items && items->data && glyphs && painter->font_style & (GTK_HTML_FONT_STYLE_UNDERLINE | GTK_HTML_FONT_STYLE_STRIKEOUT)) {
 		GList *gl, *il;
 		PangoRectangle log_rect;
-		gint width, dsc, asc;
+		gint dsc, asc;
 
 		for (gl = glyphs, il = items; gl && il; gl = gl->next, il = il->next)
 			pango_glyph_string_extents ((PangoGlyphString *) gl->data, ((PangoItem *) il->data)->analysis.font, NULL, &log_rect);
@@ -1001,6 +1042,8 @@ draw_text (HTMLPainter *painter, gint x, gint y, const gchar *text, gint len, GL
 		if (items)
 			items_destroy (items);
 	}
+
+	return width;
 }
 
 static void
@@ -1025,7 +1068,7 @@ static void
 calc_text_size (HTMLPainter *painter,
 		const gchar *text,
 		guint len,
-		GList *items, GList *glyphs,
+		GList *items, GList *glyphs, gint start_byte_offset,
 		GtkHTMLFontStyle style,
 		HTMLFontFace *face,
 		gint *width, gint *asc, gint *dsc)
@@ -1036,14 +1079,14 @@ calc_text_size (HTMLPainter *painter,
 	gdk_painter = HTML_GDK_PAINTER (painter);
 	font = html_font_manager_get_font (&painter->font_manager, face, style);
 
-	text_size (gdk_painter, (PangoFontDescription *) font->data, text, g_utf8_offset_to_pointer (text, len) - text, items, glyphs, width, asc, dsc);
+	text_size (gdk_painter, (PangoFontDescription *) font->data, text, g_utf8_offset_to_pointer (text, len) - text, items, glyphs, start_byte_offset, width, asc, dsc);
 	/* printf ("calc_text_size %d %d %d\n", *width, *asc, *dsc); */
 }
 
 static void
 calc_text_size_bytes (HTMLPainter *painter, const gchar *text,
 		      guint bytes_len,
-		      GList *items, GList *glyphs,
+		      GList *items, GList *glyphs, gint start_byte_offset,
 		      HTMLFont *font, GtkHTMLFontStyle style,
 		      gint *width, gint *asc, gint *dsc)
 {
@@ -1051,7 +1094,7 @@ calc_text_size_bytes (HTMLPainter *painter, const gchar *text,
 
 	gdk_painter = HTML_GDK_PAINTER (painter);
 
-	text_size (gdk_painter, (PangoFontDescription *) font->data, text, bytes_len, items, glyphs, width, asc, dsc);
+	text_size (gdk_painter, (PangoFontDescription *) font->data, text, bytes_len, items, glyphs, start_byte_offset, width, asc, dsc);
 
 	/* printf ("calc_text_size_bytes %d %d %d\n", *width, *asc, *dsc); */
 }
